from django.template.context import RequestContext
from django.shortcuts import render_to_response, get_object_or_404

from tendenci.core.base.http import Http403
from tendenci.core.site_settings.utils import get_setting
from tendenci.core.event_logs.models import EventLog
from tendenci.core.perms.utils import has_perm, get_query_filters, has_view_perm
from videos.models import Video, Category


def index(request, cat_slug=None, template_name="videos/list.html"):
    """
    This page lists out all videos. The order can be customized.
    Filtering by category only works if a search index is available.
    """
    category = get_object_or_404(Category, slug=cat_slug)
    filters = get_query_filters(request.user, 'videos.view_video')
    videos = Video.objects.filter(filters).distinct()
    if request.user.is_authenticated():
        videos = videos.select_related()
    if cat_slug:
        videos = videos.filter(category__slug=cat_slug)
    videos = videos.order_by('-ordering', '-create_dt')

    categories = Category.objects.all()

    EventLog.objects.log()

    return render_to_response(template_name, locals(), 
        context_instance=RequestContext(request))
            
def search(request, cat_slug=None, template_name="videos/list.html"):
    """
    This page lists out all videos. The order can be customized.
    If a search index is available, this page will also
    have the option to search through videos.
    """
    has_index = get_setting('site', 'global', 'searchindex')
    query = request.GET.get('q', None)

    categories = Category.objects.all()
    cat = request.GET.get('cat', cat_slug)

    if has_index and query:
        if cat:
            cat_query = 'cat:%s' % cat
            query = ' '.join([query, cat_query])
            categories = Category.objects.filter(slug=cat)
            category = None
            if categories:
                category = category[0]
        videos = Video.objects.search(query, user=request.user)
    else:
        filters = get_query_filters(request.user, 'videos.view_video')
        videos = Video.objects.filter(filters).distinct()
        if request.user.is_authenticated():
            videos = videos.select_related()
        videos = videos.order_by('-ordering', '-create_dt')

    EventLog.objects.log()

    return render_to_response(template_name, locals(), 
        context_instance=RequestContext(request))


def detail(request, slug, template_name="videos/details.html"):
    "Video page with embed code printed"
    
    categories = Category.objects.all()
    
    video = get_object_or_404(Video, slug=slug)

    if has_perm(request.user, 'videos.view_video', video):
        EventLog.objects.log(instance=video)

        return render_to_response(template_name, {'video': video,'categories': categories}, 
            context_instance=RequestContext(request))
    else:
        raise Http403



from django import forms

from videos.models import Video
from tinymce.widgets import TinyMCE
from tendenci.core.perms.forms import TendenciBaseForm
from embedly import Embedly

# Create Embedly instance
client = Embedly("438be524153e11e18f884040d3dc5c07")

class VideoForm(TendenciBaseForm):

    description = forms.CharField(required=False,
        widget=TinyMCE(attrs={'style':'width:100%'}, 
        mce_attrs={'storme_app_label':Video._meta.app_label, 
        'storme_model':Video._meta.module_name.lower()}))

    status_detail = forms.ChoiceField(choices=(('active','Active'),('pending','Pending')))

    clear_image = forms.BooleanField(required=False)

    class Meta:
        model = Video
        fields = (
            'title',
            'slug',
            'category',
            'image',
            'video_url',
            'tags',
            'description',
            'allow_anonymous_view',
            'user_perms',
            'group_perms',
            'member_perms',
            'status',
            'status_detail',
        )

    def __init__(self, *args, **kwargs): 
        super(VideoForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['description'].widget.mce_attrs['app_instance_id'] = self.instance.pk
        else:
            self.fields['description'].widget.mce_attrs['app_instance_id'] = 0
            
    def clean_video_url(self):
        value = self.cleaned_data.get('video_url')
        # Get embedded object from URL
        obj = client.oembed(value)
        if not value:
            raise forms.ValidationError('You must enter a URL')
        if obj.error:
            raise forms.ValidationError('This url is not supported by embed.ly')
        return value

    def save(self, *args, **kwargs):
        video = super(VideoForm, self).save(*args, **kwargs)
        if self.cleaned_data['clear_image']:
            video.image.delete()
        return video


from django.contrib import admin
from django.conf import settings

from tendenci.core.perms.admin import TendenciBaseModelAdmin
from videos.models import Video, Category
from videos.forms import VideoForm


class CategoryAdmin(admin.ModelAdmin):
    prepopulated_fields = {'slug': ['name']}


class VideoAdmin(TendenciBaseModelAdmin):

    list_display = ['title', 'category', 'ordering']
    list_filter = ['category']
    list_editable = ['ordering']
    prepopulated_fields = {'slug': ['title']}
    search_fields = ['question', 'answer']
    fieldsets = (
        (None, {'fields': ('title', 'slug', 'category', 'image', 'clear_image', 'video_url', 'tags', 'description')}),
        ('Permissions', {'fields': ('allow_anonymous_view',)}),
        ('Advanced Permissions', {'classes': ('collapse',), 'fields': (
            'user_perms',
            'member_perms',
            'group_perms',
        )}),
        ('Publishing Status', {'fields': (
            'status',
            'status_detail'
        )}),
    )
    form = VideoForm
    ordering = ['-ordering']

    class Media:
        js = (
            '%sjs/global/tinymce.event_handlers.js' % settings.STATIC_URL,
            '%sjs/jquery-1.6.2.min.js' % settings.STATIC_URL,
            '%sjs/jquery-ui-1.8.17.custom.min.js' % settings.STATIC_URL,
            '%sjs/admin/admin-list-reorder.js' % settings.STATIC_URL,
        )
    
    def get_fieldsets(self, request, obj=None):
        fieldsets = super(VideoAdmin, self).get_fieldsets(request, obj)
        if not obj or (obj and not obj.image):
            fields = list(fieldsets[0][1]['fields'])
            if 'clear_image' in fields:
                fields.remove('clear_image')
                fieldsets[0][1]['fields'] = tuple(fields)
        return fieldsets

admin.site.register(Video, VideoAdmin)
admin.site.register(Category, CategoryAdmin)



from django.conf.urls.defaults import *
from videos.feeds import LatestEntriesFeed

urlpatterns = patterns('videos.views',
    url(r'^videos/$', 'search', name="video"),
    url(r'^videos/category/([^/]+)/$', 'index', name="video.category"),
    url(r'^videos/search/$', 'search', name="video.search"),
    url(r'^videos/feed/$', LatestEntriesFeed(), name='video.feed'),
    url(r'^videos/([^/]+)/$', 'detail', name="video.details"),
)


from haystack import indexes
from haystack import site
from django.utils.html import strip_tags, strip_entities

from tendenci.core.perms.indexes import TendenciBaseSearchIndex

from videos.models import Video

class VideoIndex(TendenciBaseSearchIndex):
    title = indexes.CharField(model_attr='title')
    description = indexes.CharField(model_attr='description')
    ordering = indexes.IntegerField(model_attr='ordering')
    category = indexes.CharField()

    # RSS fields
    order = indexes.DateTimeField()

    def prepare_description(self, obj):
        description = obj.description
        description = strip_tags(description)
        description = strip_entities(description)
        return description

    def prepare_order(self, obj):
        return obj.create_dt

site.register(Video, VideoIndex)



from django.db import models
from django.core.urlresolvers import reverse
from embedly import Embedly
from django.contrib.contenttypes import generic

from tendenci.core.perms.object_perms import ObjectPermission
from tagging.fields import TagField
from tendenci.core.perms.models import TendenciBaseModel
from tinymce import models as tinymce_models
from videos.managers import VideoManager

client = Embedly("438be524153e11e18f884040d3dc5c07")

class Category(models.Model):
    name = models.CharField(max_length=200, unique=True)
    slug = models.SlugField(unique=True)
    
    def __unicode__(self):
        return self.name
    
    class Meta:
        verbose_name_plural = "Categories"
    
    def get_absolute_url(self):
        return reverse('video.category', args=[self.slug])
    
class Video(TendenciBaseModel):
    """
    Videos plugin to add embedding based on video url. Uses embed.ly
    """
    title = models.CharField(max_length=200)
    slug = models.SlugField(unique=True)
    category = models.ForeignKey(Category)
    image = models.ImageField(upload_to='uploads/videos/%y/%m', blank=True)
    video_url = models.CharField(max_length=500, help_text='Youtube, Vimeo, etc..')
    description = tinymce_models.HTMLField()
    tags = TagField(blank=True, help_text='Tag 1, Tag 2, ...')
    ordering = models.IntegerField(blank=True, null=True)

    perms = generic.GenericRelation(ObjectPermission,
                                          object_id_field="object_id",
                                          content_type_field="content_type")

    objects = VideoManager()
    
    def __unicode__(self):
        return self.title
    
    def save(self, *args, **kwargs):
        model = self.__class__
        
        if self.ordering is None:
            # Append
            try:
                last = model.objects.order_by('-ordering')[0]
                self.ordering = last.ordering + 1
            except IndexError:
                # First row
                self.ordering = 0

        return super(Video, self).save(*args, **kwargs)
    
    class Meta:
        permissions = (("view_video","Can view video"),)
        ordering = ('ordering',)
    
    @models.permalink
    def get_absolute_url(self):
        return ("video.details", [self.slug])

    def embed_code(self, **kwargs):
        width = kwargs.get('width') or 600
        return get_oembed_code(self.video_url, width, 400)

    def thumbnail(self):
        return get_oembed_thumbnail(self.video_url, 600, 400)


class OembedlyCache(models.Model):
    "For better performance all oembed queries are cached in this model"
    url = models.CharField(max_length=800)
    width = models.IntegerField(db_index=True)
    height = models.IntegerField(db_index=True)
    code = models.TextField()
    thumbnail = models.CharField(max_length=800)
    
    def __unicode__(self):
        return self.url
    
    @staticmethod
    def get_thumbnail(url, width, height):
        try:
            return OembedlyCache.objects.filter(url=url, width=width, height=height)[0].thumbnail
        except IndexError:
            try:
                result = client.oembed(url, format='json', maxwidth=width, maxheight=height)
                thumbnail = result['thumbnail_url']
                code = result['html']
            except KeyError:
                return False
            except Exception, e:
                return False
            obj = OembedlyCache(url=url, width=width, height=height, thumbnail=thumbnail, code=code)
            obj.save()
            return thumbnail
        
    @staticmethod    
    def get_code(url, width, height):
        try:
            return OembedlyCache.objects.filter(url=url, width=width, height=height)[0].code
        except IndexError:
            try:
                result = client.oembed(url, format='json', maxwidth=width, maxheight=height)
                thumbnail = result['thumbnail_url']
                code = result['html']
            except KeyError:
                return 'Unable to embed code for video <a href="%s">%s</a>' % (url, url)
            except Exception, e:
                return 'Unable to embed code for video <a href="%s">%s</a><br>Error: %s' % (url, url, e) 
            obj = OembedlyCache(url=url, width=width, height=height, code=code, thumbnail=thumbnail)
            obj.save()
            return code

def get_oembed_code(url, width, height):
    return OembedlyCache.get_code(url, width, height)
    
def get_oembed_thumbnail(url, width, height):
    return OembedlyCache.get_thumbnail(url, width, height)



from setuptools import setup, find_packages

longdesc = \
'''
An addon for Tendenci for displaying embedded videos.

Supports Vimeo and YouTube embeds. Includes drag-n-drop reordering
in the Django admin area.
'''

setup(
    name='tendenci-videos',
    author='Schipul',
    author_email='programmers@schipul.com',
    version='1.0.5',
    license='GPL3',
    description='Videos addon for Tendenci',
    long_description=longdesc,
    url='https://github.com/tendenci/tendenci-videos',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
    "tendenci>=5.1",
    "Embedly>=0.4.3",
    ],
)



from tendenci.core.registry import site
from tendenci.core.registry.base import AppRegistry, lazy_reverse
from videos.models import Video


class VideoRegistry(AppRegistry):
    version = '1.0'
    author = 'Schipul - The Web Marketing Company'
    author_email = 'programmers@schipul.com'
    description = 'Add video and display them in a grid format'
    
    event_logs = {
        'video':{
            'base':('1200000','993355'),
            'add':('1200100','119933'),
            'edit':('1200200','EEDD55'),
            'delete':('1200300','AA2222'),
            'search':('1200400','CC55EE'),
            'view':('1200500','55AACC'),
        }
    }

site.register(Video, VideoRegistry)
